// Copyright (c) 2025 Tigera, Inc. All rights reserved.

// Code generated by client-gen. DO NOT EDIT.

package v3

import (
	context "context"

	projectcalicov3 "github.com/projectcalico/api/pkg/apis/projectcalico/v3"
	scheme "github.com/projectcalico/api/pkg/client/clientset_generated/clientset/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	gentype "k8s.io/client-go/gentype"
)

// FelixConfigurationsGetter has a method to return a FelixConfigurationInterface.
// A group's client should implement this interface.
type FelixConfigurationsGetter interface {
	FelixConfigurations() FelixConfigurationInterface
}

// FelixConfigurationInterface has methods to work with FelixConfiguration resources.
type FelixConfigurationInterface interface {
	Create(ctx context.Context, felixConfiguration *projectcalicov3.FelixConfiguration, opts v1.CreateOptions) (*projectcalicov3.FelixConfiguration, error)
	Update(ctx context.Context, felixConfiguration *projectcalicov3.FelixConfiguration, opts v1.UpdateOptions) (*projectcalicov3.FelixConfiguration, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*projectcalicov3.FelixConfiguration, error)
	List(ctx context.Context, opts v1.ListOptions) (*projectcalicov3.FelixConfigurationList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *projectcalicov3.FelixConfiguration, err error)
	FelixConfigurationExpansion
}

// felixConfigurations implements FelixConfigurationInterface
type felixConfigurations struct {
	*gentype.ClientWithList[*projectcalicov3.FelixConfiguration, *projectcalicov3.FelixConfigurationList]
}

// newFelixConfigurations returns a FelixConfigurations
func newFelixConfigurations(c *ProjectcalicoV3Client) *felixConfigurations {
	return &felixConfigurations{
		gentype.NewClientWithList[*projectcalicov3.FelixConfiguration, *projectcalicov3.FelixConfigurationList](
			"felixconfigurations",
			c.RESTClient(),
			scheme.ParameterCodec,
			"",
			func() *projectcalicov3.FelixConfiguration { return &projectcalicov3.FelixConfiguration{} },
			func() *projectcalicov3.FelixConfigurationList { return &projectcalicov3.FelixConfigurationList{} },
		),
	}
}
