/*
Copyright 2024 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1alpha4

const (
	// NodeService should be listed in the "supported versions"
	// array during plugin registration by a DRA plugin which provides
	// an implementation of the v1alpha3 Node service.
	//
	// This convention was introduced in Kubernetes 1.32. Older DRA
	// plugins provide the implementation without advertising it.
	//
	// For historic reasons (= a mistake...) there is a mismatch between
	// the package name and gRPC version.
	NodeService = "v1alpha3.NodeService"
)
