// Code generated by smithy-go-codegen DO NOT EDIT.

// Package route53 provides the API client, operations, and parameter types for
// Amazon Route 53.
//
// Amazon Route 53 is a highly available and scalable Domain Name System (DNS) web
// service.
//
// You can use Route 53 to:
//
//   - Register domain names.
//
// For more information, see [How domain registration works].
//
//   - Route internet traffic to the resources for your domain
//
// For more information, see [How internet traffic is routed to your website or web application].
//
//   - Check the health of your resources.
//
// For more information, see [How Route 53 checks the health of your resources].
//
// [How domain registration works]: https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/welcome-domain-registration.html
// [How Route 53 checks the health of your resources]: https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/welcome-health-checks.html
// [How internet traffic is routed to your website or web application]: https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/welcome-dns-service.html
package route53
