// Code generated by smithy-go-codegen DO NOT EDIT.

// Package autoscaling provides the API client, operations, and parameter types
// for Auto Scaling.
//
// # Amazon EC2 Auto Scaling
//
// Amazon EC2 Auto Scaling is designed to automatically launch and terminate EC2
// instances based on user-defined scaling policies, scheduled actions, and health
// checks.
//
// For more information, see the [Amazon EC2 Auto Scaling User Guide] and the [Amazon EC2 Auto Scaling API Reference].
//
// [Amazon EC2 Auto Scaling User Guide]: https://docs.aws.amazon.com/autoscaling/ec2/userguide/what-is-amazon-ec2-auto-scaling.html
// [Amazon EC2 Auto Scaling API Reference]: https://docs.aws.amazon.com/autoscaling/ec2/APIReference/Welcome.html
package autoscaling
